import React, { useEffect, useRef } from 'react';
import { s } from 'vitest/dist/env-afee91f0';

type Props = {
  children: React.ReactNode;
  className?: string;
  style?: React.CSSProperties;
  horizontal: boolean;
  divisorSize: number;
  defaultProportion?: number;
};

function convertRemToPixels(rem: number) {
  return rem * parseFloat(getComputedStyle(document.documentElement).fontSize);
}

export const Resizable = ({ children, className, style, horizontal, divisorSize, defaultProportion, ...props }: Props) => {
  const ref = useRef<HTMLDivElement>(null);
  const children2 = children as React.ReactElement[];
  const [firstSize, setFirstSize] = React.useState<number>(0);
  const [secondSize, setSecondSize] = React.useState<number>(0);
  const [dragging, setDragging] = React.useState<boolean>(false);

  useEffect(() => {
    if (ref.current) {
      const rect = ref.current.getBoundingClientRect();
      const _defaultProportion = defaultProportion || 0.5;
      if (horizontal) {
        setFirstSize(rect.width * _defaultProportion - divisorSize);
        setSecondSize(rect.width * (1 / _defaultProportion) - divisorSize);
      } else {
        setFirstSize(rect.height * _defaultProportion - divisorSize);
        setSecondSize(rect.height * (1 / _defaultProportion) - divisorSize);
      }
    }
  }, [ref.current]);

  function onMouseDown(e: React.MouseEvent<HTMLDivElement, MouseEvent>) {
    setDragging(true);
    window.addEventListener('mouseup', onMouseUp);
  }
  const onMouseUp = () => {
    setDragging(false);
    window.removeEventListener('mouseup', onMouseUp);
  };
  function onMouseMove(e: React.MouseEvent<HTMLDivElement, MouseEvent>) {
    if (dragging) {
      if (horizontal && ref.current) {
        const rect = ref.current.getBoundingClientRect();
        setFirstSize(e.clientX);
        setSecondSize(rect.width - e.clientX);
      } else {
        setFirstSize(e.clientY);
        setSecondSize(window.innerHeight - e.clientY);
      }
    }
  }
  function onTouchStart(e: React.TouchEvent<HTMLDivElement>) {
    setDragging(true);
  }
  function onTouchMove(e: React.TouchEvent<HTMLDivElement>) {}
  function onTouchEnd(e: React.TouchEvent<HTMLDivElement>) {
    setDragging(false);
  }
  function onTouchCancel(e: React.TouchEvent<HTMLDivElement>) {
    setDragging(false);
  }

  return (
    <>
      {dragging && <div className="absolute top-0 left-0 w-screen h-screen z-10 cursor-grabbing" onMouseMove={onMouseMove}></div>}
      <div className={` w-full h-full flex ${horizontal ? 'flex-row' : 'flex-col'} ${className}`} style={style} {...props} ref={ref}>
        {firstSize > 0 && (
          <>
            <div className="h-full w-full" style={horizontal ? { maxWidth: firstSize } : { maxHeight: firstSize }}>
              {children2[0]}
            </div>
            <div
              className={' ' + (horizontal ? 'cursor-col-resize' : 'cursor-row-resize')}
              style={horizontal ? { minWidth: divisorSize } : { minHeight: divisorSize }}
              onMouseDown={onMouseDown}
              onTouchStart={onTouchStart}
              onTouchMove={onTouchMove}
              onTouchEnd={onTouchEnd}
              onTouchCancel={onTouchCancel}
            ></div>
            <div className="h-full w-full" style={horizontal ? { maxWidth: secondSize } : { maxHeight: secondSize }}>
              {children2[1]}
            </div>
          </>
        )}
      </div>
    </>
  );
};
